// //////////////////////////////////////////////////////////////////////////
// Implementation file ProjectModel.cpp for class ProjectModel
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/09/23
// //////////////////////////////////////////////////////////////////////////

#include <qfileinfo.h>
#include "FileModel.h"
#include "FileModelIterator.h"
#include "ProjectModel.h"
#include "ResolvedFileModel.h"


ProjectModel::ProjectModel( const QString &projectDirectory ) :
    _resolvedFiles( new QList<ResolvedFileModel>() ),
    _fileDependencies( new QDict<FileModel>() )
{
  QFileInfo dir( projectDirectory );
  _projectDirectory = dir.absFilePath() + '/';

  _resolvedFiles->setAutoDelete( TRUE );
  _fileDependencies->setAutoDelete( TRUE );
}


ProjectModel::~ProjectModel()
{
  delete _resolvedFiles;
  delete _fileDependencies;
}


QString 
ProjectModel::absoluteProjectPath()
{
  return _projectDirectory;
}


void 
ProjectModel::addFile( const QString &relativeFileName )
{
  FileModel *file = fileDependenciesFor( _projectDirectory + 
                                         relativeFileName );
  _resolvedFiles->append( new ResolvedFileModel( this, file ) );
}


void 
ProjectModel::addProjectRelativeIncludeDirectory( const QString &includeDirectory )
{

}


FileModelIterator 
ProjectModel::files()
{
  return FileModelIterator( 
             QListIterator<ResolvedFileModel>( *_resolvedFiles ) );
}


FileModel *
ProjectModel::fileDependenciesFor( const QString &fullPathName,
                                   bool isBadFileName )
{
  QString absolutePathName = QFileInfo(fullPathName).absFilePath();
  FileModel *dependencies = _fileDependencies->find( absolutePathName );
  if ( dependencies == NULL )
  {
    dependencies = new FileModel( fullPathName, !isBadFileName );
    _fileDependencies->insert( fullPathName, dependencies );
  }
  return dependencies;
}


ResolvedFileModel *
ProjectModel::resolvedDependencyFor( ResolvedFileModel *file,
                                     FileDependency *dependency )
{
  QString includedFileName;

  if ( dependency->searchParentDirectoryFirst() )
  {
    includedFileName = SearchFileInParentFileDirectories( file,  
                                                          dependency );
  }

  if ( includedFileName.isNull() )
    includedFileName = SearchFileInEnvironment( dependency );

  bool badFileName = includedFileName.isNull();
  if ( badFileName )
    includedFileName = dependency->fileName();

  FileModel *includedFile = fileDependenciesFor( includedFileName,
                                                 badFileName );
  return new ResolvedFileModel( this, includedFile, file );
}


QString
ProjectModel::SearchFileInEnvironment( FileDependency *dependency )
{
  QString fullPathName = absolutePathFor( "Include/" + 
                                          dependency->fileName() );
  QFileInfo includeFileInfo( fullPathName );
  if ( includeFileInfo.exists() )
    return fullPathName;

  return QString::null;
}


QString
ProjectModel::SearchFileInParentFileDirectories( 
                                    ResolvedFileModel *file, 
                                    FileDependency *dependency )
{
  ResolvedFileModel *directoryFile = file;
  do
  {
    QString fullPathName = directoryFile->directoryPath() + 
                           dependency->fileName();
    QFileInfo includeFileInfo( fullPathName );
    if ( includeFileInfo.exists() )
      return fullPathName;

    if ( !directoryFile->hasParentFile() )
      return QString::null;

    directoryFile = directoryFile->parentFile();
  }
  while ( true );
}


QString 
ProjectModel::absolutePathFor( const QString &projectRelativeFileName )
{
  QFileInfo file( absoluteProjectPath() + projectRelativeFileName );
  return file.absFilePath();
}


