// //////////////////////////////////////////////////////////////////////////
// Implementation file ProjectModelTest.cpp for class ProjectModelTest
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/09/23
// //////////////////////////////////////////////////////////////////////////

#include <qfileinfo.h>
#include "ExpectedFile.h"
#include "ExpectedProject.h"
#include "FileModelIterator.h"
#include "ProjectModel.h"
#include "ProjectModelTest.h"
#include "QtCppUnitTools.h"
#include "ResolvedFileModel.h"


CPPUNIT_TEST_SUITE_REGISTRATION( ProjectModelTest );


ProjectModelTest::ProjectModelTest() : 
    CppUnit::TestCase()
{
}


ProjectModelTest::~ProjectModelTest()
{
}


void 
ProjectModelTest::setUp()
{
  _dir = "../Test/";
}


void 
ProjectModelTest::tearDown()
{
}


QString 
ProjectModelTest::makePathFor( const QString &projectDirectory,
                               const QString &relativeFileName )
{
  QFileInfo file( _dir + projectDirectory + relativeFileName );
  return file.absFilePath();
}


void 
ProjectModelTest::testAbsoluteProjectPath()
{
  ProjectModel project( _dir + "Project1/" );
  
  QFileInfo dir( _dir + "Project1/" );
  QString expectedPath = dir.absFilePath();
  if ( expectedPath.right(1) != "/" )
    expectedPath += "/";

  CPPUNIT_ASSERT( project.absoluteProjectPath().contains( '\\' ) == 0 );
  CPPUNIT_ASSERT_EQUAL( expectedPath, project.absoluteProjectPath() );
}


void 
ProjectModelTest::testOneFileAndOneDependency()
{
  QString projectDir( "Project1/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addFile( "Object.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );

  file1->addFile( new ExpectedFile( "Object.h" ) );
  project.checkSameAs( &actualProject );
}


void 
ProjectModelTest::testDirectThreeLevelDependencies()
{
  QString projectDir( "Project2/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addFile( "Object.cpp" );
  actualProject.addFile( "Attribut.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );
  ExpectedFile *file1_1 = file1->addFile( "Object.h" );
  file1_1->addFile( "Attribut.h" )->addFile( "string" );
  file1_1->addFile( "vector" );

  ExpectedFile *file2 = new ExpectedFile( "Attribut.cpp" );
  project.addFile( file2 );
  file2->addFile( "Attribut.h" )->addFile( "string" );

  project.checkSameAs( &actualProject );
}


void 
ProjectModelTest::testSearchParentDirectory()
{
  QString projectDir( "Project3/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addFile( "Object.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );
  file1->addFile( "Headers/Object.h" )
         ->addFile( "Tools/vector" )
           ->addFile( "Headers/string" );

  project.checkSameAs( &actualProject );
}


void 
ProjectModelTest::testSearchIncludePathDependencies()
{
  QString projectDir( "Project4/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addProjectRelativeIncludeDirectory( "Include" );
  actualProject.addFile( "Object.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );
  file1->addFile( "Include/Stl/string" );

  project.checkSameAs( &actualProject );
}


void 
ProjectModelTest::testSearchParentBeforeIncludePath()
{
  QString projectDir( "Project5/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addProjectRelativeIncludeDirectory( "Include" );
  actualProject.addFile( "Object.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );
  file1->addFile( "string" );

  project.checkSameAs( &actualProject );
}


void 
ProjectModelTest::testSearchParentOnlyIfQuoted()
{
  QString projectDir( "Project6/" );
  ProjectModel actualProject( _dir + projectDir );
  actualProject.addProjectRelativeIncludeDirectory( "Include" );
  actualProject.addFile( "Object.cpp" );

  ExpectedProject project( _dir + projectDir );
  ExpectedFile *file1 = new ExpectedFile( "Object.cpp" );
  project.addFile( file1 );
  file1->addFile( "Include/string" );

  project.checkSameAs( &actualProject );
}
